package org.apidesign.apifest08.currency;

import java.util.*;

/** This is the skeleton class for your API. You need to make it public, so
 * it is accessible to your client code (currently in Task1Test.java) file.
 * <p>
 * Feel free to create additional classes or rename this one, just keep all
 * the API and its implementation in this package. Do not spread it outside
 * to other packages.
 *
 * The converter will usually work internally with Currency class, to be more type-safe as String can be used
 * for almost anything. But methods taking strings as parameters are provided for convenience.
 */
public interface CurrencyConverter {

	/**
	 * This is convenience method for convert(Currency.getInstance(from), Currency.getInstance(to)).
	 *
	 * @param value that should be converted form one currency to the other
	 * @param from ISO-4217 code of the currency of the value provided
	 * @param to ISO-4212 code of the currency to which the value should be converted
	 * @return value expressed in the target value
	 * @throws IllegalArgumentException if any of the arguments is not a valid ISO code
	 * @throws CurrencyConversionException if the conversion cannot be performed with desired parameters,
	 * for example the exchange rates are not current, connection to exchange rates provider is not available
	 * @throws NullPointerException if any of the specified currency ISO codes is null
	 */
	// this method is provided to ensure future compatibility for converters supporting more than 2 currencies
	//		- simpler methods with fewer arguments would make using such converters less intuitive
	double convert(double value, /*@NotNull*/ String from, /*@NotNull*/ String to)
			throws CurrencyConversionException, NullPointerException, IllegalArgumentException;

	/**
	 * Converts the specified value from one currency (from) to target currency (to).
	 *
	 * @param value that should be converted form one currency to the other
	 * @param from ISO-4217 code of the currency of the value provided
	 * @param to ISO-4212 code of the currency to which the value should be converted
	 * @return value expressed in the target value
	 * @throws IllegalArgumentException if any of the arguments is not a valid ISO code
	 * @throws CurrencyConversionException if the conversion cannot be performed with desired parameters,
	 * for example the exchange rates are not current, connection to exchange rates provider is not available
	 * @throws NullPointerException if any of the specified currency ISO codes is null
	 */
	double convert(double value, /*@NotNull*/ Currency from, /*@NotNull*/ Currency to)
			throws CurrencyConversionException, NullPointerException;
}
